%=================================================================
%buildGBCHcyclic.m
%build Generatormatrix of a systematic BCH code
%The calling syntax is:
%		[G,H,gn,dmin] = buildGBCHcyclic(n,k,gen,vdmin)
%Inputs:
%   n,k: code parameters, gen in octal notation, verifica dmin
%Outputs:
%   Generator matrix, Parity check matrix
% n=7;  k=4;  t=1; gen=[1 3];
% n=15; k=11; t=1; gen=[2 3];
% n=15; k=7;  t=2; gen=[7 2 1];
% n=15; k=5;  t=3; gen=[2 4 6 7];
% n=31; k=26; t=1; gen=[4 5];
% n=31; k=21; t=2; gen=[3 5 5 1];
% n=31; k=16; t=3; gen=[1 0 7 6 5 7];
% n=31; k=11; t=5; gen=[5 4 2 3 3 2 5];
% n=31; k=6;  t=7; gen=[3 1 3 3 6 5 0 4 7];
% n=63; k=57; t=1; gen=[1 0 3];
% n=63; k=51; t=2; gen=[1 2 4 7 1];
% n=63; k=45; t=3; gen=[1 7 0 1 3 1 7];
% n=63; k=39; t=4; gen=[1 6 6 6 2 3 5 6 7];
% n=63; k=36; t=5; gen=[1 0 3 3 5 0 0 4 2 3];
% n=63; k=30; t=6; gen=[1 5 7 4 6 4 1 6 5 5 4 7];
% n=63; k=24; t=7; gen=[1 7 3 2 3 2 6 0 4 0 4 4 4 1];
% n=63; k=18; t=10;gen=[1 3 6 3 0 2 6 5 1 2 3 5 1 7 2 5];
% n=127;k=22; t=23;gen=[1 2 3 3 7 6 0 7 0 4 0 4 7 2 2 5 2 2 4 3 5 4 4 5 6 2 6 6 3 7 6 4 7 0 4 3];
%=================================================================
function [G,H,gn,dmin] = buildGBCHcyclic(n,k,gen,vdmin)
G=zeros(k,n);
H=zeros(n-k,n);
for i=1:length(gen)
    mpp=fliplr(gen(i));
    for j=1:3
        mp0(j)=rem(mpp,2);
        mpp=(mpp-mp0(j))/2;
    end
    mpb(3*i-2:3*i)=fliplr(mp0);
end
if mpb(length(mpb))==0
    disp('wrong generator');
    return
end
L=n-k+1;
gn=mpb(1+rem(3-rem(L,3),3):length(mpb));
%keyboard
G(k,:)=[zeros(1,k-1) gn];
for i=k-1:-1:1
    G(i,:)=[G(i+1,2:n) 0];
    if G(i,k)==1
        G(i,:)=rem(G(i,:)+G(k,:),2);
    end
end
H(:,1:k)=G(:,k+1:n)';
H(:,k+1:n)=eye(n-k);
dmin=1000;
if vdmin
    for j=1:2^k-1; 
        u=zeros(1,k);
        j0=j;
        for i=1:k, 
            u(i)=rem(j0,2);
            j0=(j0-u(i))/2; 
        end; 
        c=u*G;
        dmin=min(dmin,sum(c)); 
    end
end
